local VERSION = tonumber(("$Revision: 40989 $"):match("%d+"))

CowTip = AceLibrary("AceAddon-2.0"):new("AceDB-2.0", "AceEvent-2.0", "AceConsole-2.0", "FuBarPlugin-2.0", "AceHook-2.1")
local CowTip, self = CowTip, CowTip
CowTip.version = "r" .. VERSION
CowTip.revision = VERSION
CowTip.date = ("$Date: 2007-06-21 02:36:07 -0400 (Thu, 21 Jun 2007) $"):match("%d%d%d%d%-%d%d%-%d%d")

local SharedMedia = AceLibrary("SharedMedia-1.0")
local DogTag = AceLibrary("DogTag-1.0")

local _G = _G
local GameTooltip = _G.GameTooltip
local UIParent = _G.UIParent
local GetCursorPosition = _G.GetCursorPosition
local UnitExists = _G.UnitExists

CowTip:RegisterDB("CowTipDB")
CowTip:RegisterDefaults('profile', {
	scale = 1,
	border = "Blizzard Tooltip",
	unitAnchor = "CURSOR",
	frameAnchor = "BOTTOMRIGHT",
	lines = {
		lefts = {
			['*'] = "",
			"[Name]", -- [1]
			"Target:", -- [2]
			"Guild:", -- [3]
			"Guild rank:", -- [4]
			"Level:", -- [5]
			"Race:", -- [6]
			"Class:", -- [7]
			"Faction:", -- [8]
			"Status:", -- [9]
			"Health:", -- [10]
			"[~HasNoMP?Text([TypePower]:)]", -- [11]
			"[CurXP:HideZero?Text(Experience:)]", -- [12]
			"[Threat:HideZero?Text(Threat:)]", -- [13]
			"Happiness:", -- [14]
		},
		rights = {
			['*'] = "",
			"", -- [1]
			"[Target]", -- [2]
			"[Guild:Angle]", -- [3]
			"[GuildRank]", -- [4]
			"[Classification] [Level]", -- [5]
			"[SmartRace]", -- [6]
			"[SmartClass]", -- [7]
			"[Faction]", -- [8]
			"[Status]", -- [9]
			"[FractionalHP:Short] [PercentHP:Percent:Paren]", -- [10]
			"[FractionalMP:Short] [PercentMP:Percent:Paren]", -- [11]
			"[FractionalXP:Short] [PercentXP:Percent:Paren]", -- [12]
			"[FractionalThreat:Short] [PercentThreat:Percent:Paren]", -- [13]
			"[HappyText]", -- [14]
		}
	}
})

-- FuBar configuration
CowTip.hasIcon = true
CowTip.hideWithoutStandby = true
CowTip.defaultPosition = "MINIMAP"
CowTip.defaultMinimapPosition = 230
CowTip.hasNoText = true
CowTip.hasNoColor = true
CowTip.cannotDetachTooltip = true
CowTip.independentProfile = true
CowTip.blizzardTooltip = true

function CowTip:OnInitialize()
	self:RegisterChatCommand('/CowTip', '/Cow', {
		type = 'execute',
		func = function()
			self:OpenMenu()
		end
	})
end

function CowTip:OnEnable()
	self:RegisterEvent("PLAYER_ENTERING_WORLD")
	
	self:HookScript(GameTooltip, "OnShow", "GameTooltip_OnShow")
	self:HookScript(GameTooltip, "OnHide", "GameTooltip_OnHide")
	self:HookScript(GameTooltip, "OnTooltipSetUnit", "GameTooltip_OnTooltipSetUnit")
	
	self:Hook("GameTooltip_SetDefaultAnchor", "GameTooltip_SetDefaultAnchor", true)
	
	self:SetScale(nil)
	self:SetBorder(nil)
end

function CowTip:OnDisable()
	self:GameTooltip_OnHide(GameTooltip)
	
	self:SetScale(nil)
	self:SetBorder(nil)
end

function CowTip:PLAYER_ENTERING_WORLD()
	-- rescale the tooltip
	self:SetScale(self.db.profile.scale)
end

local lastNumRegistered = 0
local currentUnit
function CowTip:GameTooltip_OnTooltipSetUnit(GameTooltip, ...)
--	self.hooks[this].OnTooltipSetUnit(GameTooltip, unit, ...)
--	do return end
	
	local name, unit = GameTooltip:GetUnit()
	currentUnit = unit
	local mouseover = unit == "mouseover"
	
	GameTooltip:ClearLines()
	local db = self.db.profile.lines
	local lefts, rights = db.lefts, db.rights
	local num = 0
	for i = 1, #lefts do
		local left, right = lefts[i], rights[i]
		local left_ev, right_ev
		if left == "" then
			left = nil
		else
			left_ev = DogTag:Evaluate(unit, left)
			if left_ev == "" then
				left_ev = nil
			end
		end
		if right == "" then
			right = nil
		else
			right_ev = DogTag:Evaluate(unit, right)
			if right_ev == "" then
				right_ev = nil
			end
		end
		if right then
			if right_ev and left and left_ev then
				num = num + 1
				GameTooltip:AddDoubleLine(left_ev or '', right_ev, 1, 1, 1, 1, 1, 1)
				local leftFS = _G["GameTooltipTextLeft" .. num]
				local rightFS = _G["GameTooltipTextRight" .. num]
				DogTag:RegisterFontString(leftFS, GameTooltip, unit, left or '')
				DogTag:RegisterFontString(rightFS, GameTooltip, unit, right)
			end
		elseif left then
			if left_ev then
				num = num + 1
				GameTooltip:AddLine(left_ev, 1, 1, 1, 1, 1, 1)
				local leftFS = _G["GameTooltipTextLeft" .. num]
				DogTag:RegisterFontString(leftFS, GameTooltip, unit, left)
			end
		end
	end
	lastNumRegistered = num
end

local currentOwner = nil

local function ReanchorUnitTooltip(this)
	this:ClearAllPoints()
	local anchor = CowTip.db.profile.unitAnchor
	if anchor == "CURSOR" then
		if currentUnit and not UnitExists(currentUnit) then
			this:Hide()
			return
		end
		local x, y = GetCursorPosition()
		local scale = this:GetEffectiveScale()
		x , y = x / scale, y / scale
		this:SetPoint("BOTTOM", UIParent, "BOTTOMLEFT", x, y)
	else
		this:SetPoint(anchor, UIParent, anchor, 0, 0)
	end
end

function CowTip:GameTooltip_SetDefaultAnchor(this, owner, ...)
	currentOwner = owner
	self.hooks.GameTooltip_SetDefaultAnchor(this, owner, ...)
	
	local db = self.db.profile
	
	local anchor
	if owner == UIParent then -- world character
		anchor = db.unitAnchor
	else -- frame
		anchor = db.frameAnchor
	end
	if anchor == "CURSOR" then
		this:SetOwner(owner, "ANCHOR_CURSOR")
	else
		this:SetOwner(owner, "ANCHOR_NONE")
		this:ClearAllPoints()
		this:SetPoint(anchor, UIParent, anchor, 0, 0)
	end
end

function CowTip:GameTooltip_OnShow(this, ...)
	self.hooks[this].OnShow(this, ...)
	
--	this:SetScript("OnUpdate", ReanchorUnitTooltip)
end

function CowTip:GameTooltip_OnHide(this, ...)
	for i = 1, lastNumRegistered do
		DogTag:UnregisterFontString(_G["GameTooltipTextLeft" .. i])
		DogTag:UnregisterFontString(_G["GameTooltipTextRight" .. i])
	end
	lastNumRegistered = 0
	
	currentUnit = nil
	
--	this:SetScript("OnUpdate", nil)
	
	self.hooks[this].OnHide(this, ...)
end

function CowTip:SetScale(value)
	if value then
		self.db.profile.scale = value
	else
		value = self.db.profile.scale
	end
	if not self:IsActive() then
		value = 1
	end
	
	GameTooltip:SetScale(value)
end

function CowTip:SetBorder(value)
	if value then
		self.db.profile.border = value
	else
		value = self.db.profile.border
	end
	if not self:IsActive() then
		value = "Blizzard Tooltip"
	end
	
	GameTooltip:SetBackdrop({
		bgFile = "Interface/Tooltips/UI-Tooltip-Background",
		edgeFile = SharedMedia:Fetch('border', value),
		tile = true,
		tileSize = 16,
		edgeSize = 16,
		insets = {
				left = 5,
				right = 5,
				top = 5,
				bottom = 5,
			}
		} 
	)
	GameTooltip:SetBackdropColor(0, 0, 0, 1)
end

function CowTip:SetUnitAnchor(value)
	self.db.profile.unitAnchor = value
end

function CowTip:SetFrameAnchor(value)
	self.db.profile.frameAnchor = value
end

local anchorValidate = {
	CURSOR = "Cursor",
	TOPLEFT = "Screen, Top-left",
	TOP = "Screen, Top",
	TOPRIGHT = "Screen, Top-right",
	LEFT = "Screen, Left",
	RIGHT = "Screen, Right",
	BOTTOMLEFT = "Screen, Bottom-left",
	BOTTOM = "Screen, Bottom",
	BOTTOMRIGHT = "Screen, Bottom-right",
	CENTER = "Screen, Middle",
}

CowTip.options = {
	type = 'group',
	handler = CowTip,
	args = {
		lines = {
			name = "Lines",
			desc = "Here you can describe the text on the assorted lines of the tooltip.",
			type = 'group',
			args = {
				
			}
		},
		size = {
			name = "Size",
			desc = "Set how large the tooltip is",
			type = 'range',
			min = 0.25,
			max = 4,
			step = 0.01,
			bigStep = 0.05,
			get = function()
				return CowTip.db.profile.scale
			end,
			set = "SetScale"
		},
		border = {
			name = "Border",
			desc = "Change the border type.",
			type = 'text',
			validate = SharedMedia:List('border'),
			get = function()
				return CowTip.db.profile.border
			end,
			set = "SetBorder",
		},
		position = {
			name = "Position",
			desc = "Change where the tooltip is located.",
			type = 'group',
			args = {
				unitAnchor = {
					type = 'text',
					name = "Unit anchor",
					desc = "The anchor with which unit mouseover tooltips are located.",
					get = function()
						return CowTip.db.profile.unitAnchor
					end,
					set = "SetUnitAnchor",
					validate = anchorValidate,
				},
				frameAnchor = {
					type = 'text',
					name = "Frame anchor",
					desc = "The anchor with which tooltips are located when hovering over frames.",
					get = function()
						return CowTip.db.profile.frameAnchor
					end,
					set = "SetFrameAnchor",
					validate = anchorValidate,
				},
			}
		}
	}
}
local function getLeft(i)
	return CowTip.db.profile.lines.lefts[i]
end
local function setLeft(i, value)
	CowTip.db.profile.lines.lefts[i] = value
end
local function getRight(i)
	return CowTip.db.profile.lines.rights[i]
end
local function setRight(i, value)
	CowTip.db.profile.lines.rights[i] = value
end
for i = 1, 30 do
	CowTip.options.args.lines.args[i] = {
		name = ("Line #%d"):format(i),
		desc = ("Edit line #%d"):format(i),
		type = 'group',
		args = {
			left = {
				name = "Left",
				desc = "Left",
				type = 'text',
				usage = "DogTag-1.0 tags. See http://www.wowace.com/wiki/DogTag-1.0",
				get = getLeft,
				set = setLeft,
				passValue = i,
			},
			right = {
				name = "Right",
				desc = "Right",
				type = 'text',
				usage = "DogTag-1.0 tags. See http://www.wowace.com/wiki/DogTag-1.0",
				get = getRight,
				set = setRight,
				passValue = i,
			}
		},
		order = i,
	}
end
CowTip.OnMenuRequest = CowTip.options

function CowTip:OnTooltipUpdate()
	GameTooltip:AddLine("CowTip")
	GameTooltip:AddLine(" ")
	--[[
	GameTooltip:AddDoubleLine(LOCKED .. ":", (self.db.profile.settings.locked and YES or NO), 1, 1, 0, 1, 1, 1)
	local text
	if configMode == "solo" then
		text = L["Solo"]
	elseif configMode == "party" then
		text = L["Party"]
	elseif configMode == "raid" then
		text = L["Raid"]
	else
		text = L["Off"]
	end
	GameTooltip:AddDoubleLine(L["Configuration mode"] .. ":", text, 1, 1, 0, 1, 1, 1)
	GameTooltip:AddLine(" ")
	GameTooltip:AddLine(L["|cffffff00Click|r to lock/unlock unit frames."], 0.2, 1, 0.2)
	GameTooltip:AddLine(L["|cffffff00Shift-Click|r to change configuration mode."], 0.2, 1, 0.2)
	]]
	GameTooltip:AddLine("|cffffff00Right-Click|r to change settings.", 0.2, 1, 0.2)
end
